Package: affiner
Type: Package
Title: A Finer Way to Render 3D Illustrated Objects in 'grid' Using
        Affine Transformations
Version: 0.3.1
Authors@R: c(person("Trevor L.", "Davis", role = c("aut", "cre"),
                    email = "trevor.l.davis@gmail.com",
                    comment = c(ORCID = "0000-0001-6341-4639")))
Description: Dilate, permute, project, reflect, rotate, shear, and translate 2D and 3D points.  Supports parallel projections including oblique projections such as the cabinet projection as well as axonometric projections such as the isometric projection.  Use 'grid's "affine transformation" feature to render illustrated flat surfaces.
URL: https://trevorldavis.com/R/affiner/
BugReports: https://github.com/trevorld/affiner/issues
License: MIT + file LICENSE
Depends: R (>= 3.6.0)
Imports: graphics, grDevices, grid, R6, utils
Suggests: aRtsy, ggplot2, gridpattern, gtable, knitr, ragg (>= 1.3.3),
        rgl, rlang, rmarkdown, stats, testthat (>= 3.0.0), vdiffr,
        withr
VignetteBuilder: knitr, ragg, rmarkdown
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-10 04:57:18 UTC; trevorld
Author: Trevor L. Davis [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6341-4639>)
Maintainer: Trevor L. Davis <trevor.l.davis@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-10 06:20:02 UTC
Built: R 4.6.0; ; 2026-05-10 08:20:04 UTC; unix
