# Generated by roxygen2: do not edit by hand

S3method("!=",Coord1D)
S3method("!=",Coord2D)
S3method("!=",Coord3D)
S3method("*",Coord1D)
S3method("*",Coord2D)
S3method("*",Coord3D)
S3method("+",Coord1D)
S3method("+",Coord2D)
S3method("+",Coord3D)
S3method("-",Coord1D)
S3method("-",Coord2D)
S3method("-",Coord3D)
S3method("/",Coord1D)
S3method("/",Coord2D)
S3method("/",Coord3D)
S3method("==",Coord1D)
S3method("==",Coord2D)
S3method("==",Coord3D)
S3method("[",Coord1D)
S3method("[",Coord2D)
S3method("[",Coord3D)
S3method("[",Ellipse2D)
S3method("[",Line2D)
S3method("[",Plane3D)
S3method("[",Point1D)
S3method("[",Segment2D)
S3method("[",angle)
S3method(Arg,Coord2D)
S3method(Arg,angle)
S3method(Complex,at_matrix)
S3method(Conj,Coord2D)
S3method(Im,Coord2D)
S3method(Math,at_matrix)
S3method(Mod,Coord2D)
S3method(Ops,angle)
S3method(Ops,at_matrix)
S3method(Re,Coord2D)
S3method(abs,Coord1D)
S3method(abs,Coord2D)
S3method(abs,Coord3D)
S3method(abs,angle)
S3method(as.complex,Coord2D)
S3method(as.complex,angle)
S3method(as.data.frame,Coord1D)
S3method(as.data.frame,Coord2D)
S3method(as.data.frame,Coord3D)
S3method(as.data.frame,Ellipse2D)
S3method(as.data.frame,Line2D)
S3method(as.data.frame,Plane3D)
S3method(as.data.frame,Point1D)
S3method(as.double,angle)
S3method(as.list,Coord1D)
S3method(as.list,Coord2D)
S3method(as.list,Coord3D)
S3method(as.list,Ellipse2D)
S3method(as.list,Line2D)
S3method(as.list,Plane3D)
S3method(as.list,Point1D)
S3method(as.matrix,Coord1D)
S3method(as.matrix,Coord2D)
S3method(as.matrix,Coord3D)
S3method(as.matrix,at_matrix)
S3method(as_angle,Coord2D)
S3method(as_angle,Coord3D)
S3method(as_angle,Line2D)
S3method(as_angle,Plane3D)
S3method(as_angle,angle)
S3method(as_angle,character)
S3method(as_angle,complex)
S3method(as_angle,numeric)
S3method(as_coord1d,Coord1D)
S3method(as_coord1d,Coord2D)
S3method(as_coord1d,Point1D)
S3method(as_coord1d,character)
S3method(as_coord1d,data.frame)
S3method(as_coord1d,list)
S3method(as_coord1d,matrix)
S3method(as_coord1d,numeric)
S3method(as_coord2d,Coord2D)
S3method(as_coord2d,Coord3D)
S3method(as_coord2d,angle)
S3method(as_coord2d,character)
S3method(as_coord2d,complex)
S3method(as_coord2d,data.frame)
S3method(as_coord2d,list)
S3method(as_coord2d,matrix)
S3method(as_coord2d,numeric)
S3method(as_coord3d,Coord2D)
S3method(as_coord3d,Coord3D)
S3method(as_coord3d,angle)
S3method(as_coord3d,character)
S3method(as_coord3d,data.frame)
S3method(as_coord3d,list)
S3method(as_coord3d,matrix)
S3method(as_coord3d,numeric)
S3method(as_ellipse2d,Coord2D)
S3method(as_ellipse2d,numeric)
S3method(as_line2d,Coord2D)
S3method(as_line2d,Line2D)
S3method(as_line2d,Point1D)
S3method(as_line2d,angle)
S3method(as_line2d,character)
S3method(as_line2d,numeric)
S3method(as_plane3d,Coord3D)
S3method(as_plane3d,Line2D)
S3method(as_plane3d,Plane3D)
S3method(as_plane3d,Point1D)
S3method(as_plane3d,character)
S3method(as_plane3d,numeric)
S3method(as_point1d,Coord1D)
S3method(as_point1d,Point1D)
S3method(as_point1d,character)
S3method(as_point1d,numeric)
S3method(as_polygon2d,Coord2D)
S3method(as_polygon2d,Ellipse2D)
S3method(as_polygon2d,Polygon2D)
S3method(as_polygon2d,numeric)
S3method(as_segment2d,Coord2D)
S3method(as_segment2d,Polygon2D)
S3method(as_transform1d,default)
S3method(as_transform1d,transform1d)
S3method(as_transform2d,default)
S3method(as_transform2d,transform2d)
S3method(as_transform3d,default)
S3method(as_transform3d,transform3d)
S3method(c,Coord1D)
S3method(c,Coord2D)
S3method(c,Coord3D)
S3method(c,Line2D)
S3method(c,Plane3D)
S3method(c,Point1D)
S3method(c,angle)
S3method(ceiling,Coord1D)
S3method(ceiling,Coord2D)
S3method(ceiling,Coord3D)
S3method(convex_hull2d,Coord2D)
S3method(convex_hull2d,Polygon2D)
S3method(cos,angle)
S3method(floor,Coord1D)
S3method(floor,Coord2D)
S3method(floor,Coord3D)
S3method(format,angle)
S3method(ggplot2::autolayer,Coord1D)
S3method(ggplot2::autolayer,Coord2D)
S3method(ggplot2::autolayer,Ellipse2D)
S3method(ggplot2::autolayer,Line2D)
S3method(ggplot2::autolayer,Point1D)
S3method(ggplot2::autolayer,Polygon2D)
S3method(ggplot2::autolayer,Segment2D)
S3method(has_intersection,Line2D)
S3method(has_intersection,Plane3D)
S3method(has_intersection,Point1D)
S3method(has_intersection,default)
S3method(has_overlap2d,Ellipse2D)
S3method(has_overlap2d,Polygon2D)
S3method(intersection,Coord1D)
S3method(intersection,Coord2D)
S3method(intersection,Coord3D)
S3method(intersection,Ellipse2D)
S3method(intersection,Line2D)
S3method(intersection,Plane3D)
S3method(intersection,Point1D)
S3method(is.finite,Coord1D)
S3method(is.finite,Coord2D)
S3method(is.finite,Coord3D)
S3method(is.finite,Line2D)
S3method(is.finite,Plane3D)
S3method(is.finite,Point1D)
S3method(is.infinite,Coord1D)
S3method(is.infinite,Coord2D)
S3method(is.infinite,Coord3D)
S3method(is.infinite,Line2D)
S3method(is.infinite,Plane3D)
S3method(is.infinite,Point1D)
S3method(is.na,Coord1D)
S3method(is.na,Coord2D)
S3method(is.na,Coord3D)
S3method(is.na,Line2D)
S3method(is.na,Plane3D)
S3method(is.na,Point1D)
S3method(is.nan,Coord1D)
S3method(is.nan,Coord2D)
S3method(is.nan,Coord3D)
S3method(is.nan,Line2D)
S3method(is.nan,Plane3D)
S3method(is.nan,Point1D)
S3method(is_congruent,angle)
S3method(is_congruent,numeric)
S3method(is_degenerate,Coord1D)
S3method(is_degenerate,Coord2D)
S3method(is_degenerate,Coord3D)
S3method(is_degenerate,Line2D)
S3method(is_degenerate,Plane3D)
S3method(is_degenerate,Point1D)
S3method(is_equivalent,Coord1D)
S3method(is_equivalent,Coord2D)
S3method(is_equivalent,Coord3D)
S3method(is_equivalent,Line2D)
S3method(is_equivalent,Plane3D)
S3method(is_equivalent,Point1D)
S3method(is_equivalent,angle)
S3method(is_equivalent,numeric)
S3method(is_parallel,Line2D)
S3method(is_parallel,Plane3D)
S3method(length,Coord1D)
S3method(length,Coord2D)
S3method(length,Coord3D)
S3method(length,Line2D)
S3method(length,Plane3D)
S3method(length,Point1D)
S3method(lines,Coord2D)
S3method(lines,Ellipse2D)
S3method(lines,Line2D)
S3method(lines,Point1D)
S3method(lines,Polygon2D)
S3method(lines,Segment2D)
S3method(makeContent,affine)
S3method(makeContent,isocube)
S3method(mean,Coord1D)
S3method(mean,Coord2D)
S3method(mean,Coord3D)
S3method(normal2d,Coord2D)
S3method(normal2d,Line2D)
S3method(normal3d,Coord3D)
S3method(normal3d,Plane3D)
S3method(normal3d,character)
S3method(painter_depth,Coord2D)
S3method(painter_depth,Coord3D)
S3method(painter_depth,Polygon2D)
S3method(painter_depth,Segment2D)
S3method(painter_order,Coord2D)
S3method(painter_order,Coord3D)
S3method(painter_order,Polygon2D)
S3method(plot,Coord1D)
S3method(plot,Coord2D)
S3method(plot,Ellipse2D)
S3method(plot,Polygon2D)
S3method(plot,Segment2D)
S3method(points,Coord1D)
S3method(points,Coord2D)
S3method(print,angle)
S3method(range,Coord1D)
S3method(range,Coord2D)
S3method(range,Coord3D)
S3method(rep,Coord1D)
S3method(rep,Coord2D)
S3method(rep,Coord3D)
S3method(rep,Line2D)
S3method(rep,Plane3D)
S3method(rep,Point1D)
S3method(rep,Segment2D)
S3method(rep,angle)
S3method(rgl::plot3d,Coord3D)
S3method(rgl::plot3d,Plane3D)
S3method(round,Coord1D)
S3method(round,Coord2D)
S3method(round,Coord3D)
S3method(signif,Coord1D)
S3method(signif,Coord2D)
S3method(signif,Coord3D)
S3method(sin,angle)
S3method(solve,at_matrix)
S3method(sort,Coord2D)
S3method(sort,Coord3D)
S3method(standardize,Line2D)
S3method(standardize,Plane3D)
S3method(standardize,Point1D)
S3method(sum,Coord1D)
S3method(sum,Coord2D)
S3method(sum,Coord3D)
S3method(t,at_matrix)
S3method(tan,angle)
S3method(trunc,Coord1D)
S3method(trunc,Coord2D)
S3method(trunc,Coord3D)
export("angular_unit<-")
export(Coord1D)
export(Coord2D)
export(Coord3D)
export(Ellipse2D)
export(Line2D)
export(Plane3D)
export(Point1D)
export(Polygon2D)
export(Segment2D)
export(affineGrob)
export(affine_settings)
export(affiner_options)
export(angle)
export(angular_unit)
export(arccosecant)
export(arccosine)
export(arccotangent)
export(arcsecant)
export(arcsine)
export(arctangent)
export(as_angle)
export(as_coord1d)
export(as_coord2d)
export(as_coord3d)
export(as_ellipse2d)
export(as_line2d)
export(as_plane3d)
export(as_point1d)
export(as_polygon2d)
export(as_segment2d)
export(as_transform1d)
export(as_transform2d)
export(as_transform3d)
export(convex_hull2d)
export(cosecant)
export(cosine)
export(cotangent)
export(cross_product3d)
export(degrees)
export(distance1d)
export(distance2d)
export(distance3d)
export(dot_product1d)
export(dot_product2d)
export(dot_product3d)
export(gradians)
export(grid.affine)
export(grid.isocube)
export(has_intersection)
export(has_overlap2d)
export(intersection)
export(is_angle)
export(is_congruent)
export(is_coord1d)
export(is_coord2d)
export(is_coord3d)
export(is_ellipse2d)
export(is_equivalent)
export(is_line2d)
export(is_parallel)
export(is_plane3d)
export(is_point1d)
export(is_polygon2d)
export(is_segment2d)
export(is_transform1d)
export(is_transform2d)
export(is_transform3d)
export(isocubeGrob)
export(isotoxal_2ngon_inner_radius)
export(isotoxal_2ngon_polygon2d)
export(normal2d)
export(normal3d)
export(painter_depth)
export(painter_order)
export(permute2d)
export(permute3d)
export(pi_radians)
export(project1d)
export(project2d)
export(project3d)
export(radians)
export(rectangle_polygon2d)
export(reflect1d)
export(reflect2d)
export(reflect3d)
export(regular_ngon_polygon2d)
export(rotate2d)
export(rotate3d)
export(rotate3d_to_AA)
export(scale1d)
export(scale2d)
export(scale3d)
export(secant)
export(shear2d)
export(shear3d)
export(sine)
export(star_inner_radius)
export(star_polygon2d)
export(tangent)
export(transform1d)
export(transform2d)
export(transform3d)
export(translate1d)
export(translate2d)
export(translate3d)
export(turns)
if (getRversion() >= "4.3.0") {
  S3method("%*%",Coord1D)
}
if (getRversion() >= "4.3.0") {
  S3method("%*%",Coord2D)
}
if (getRversion() >= "4.3.0") {
  S3method("%*%",Coord3D)
}
if (getRversion() >= "4.3.0") {
  S3method("%*%",at_matrix)
}
if (getRversion() >= "4.4.0") {
  S3method("crossprod",Coord3D)
}
importFrom(R6,R6Class)
importFrom(graphics,lines)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(grid,makeContent)
importFrom(utils,hasName)
